//@line 36 "/builds/moz2_slave/release-mozilla-1.9.1-linux_repack_5/build/l10n-mozilla-1.9.1/ro/browser/firefox-l10n.js"

//@line 38 "/builds/moz2_slave/release-mozilla-1.9.1-linux_repack_5/build/l10n-mozilla-1.9.1/ro/browser/firefox-l10n.js"

pref("general.useragent.locale", "ro");
